"""empty message

Revision ID: f46112e923d8
Revises:
Create Date: 2023-02-27 09:43:42.498907

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "f46112e923d8"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "users",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("first_name", sa.String(length=16), nullable=False),
        sa.Column("middle_name", sa.String(length=16), nullable=True),
        sa.Column("last_name", sa.String(length=16), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("users")
    # ### end Alembic commands ###



import logging

from functools import wraps

from sqlalchemy import create_engine
from sqlalchemy.sql.dml import Insert, Update, Delete
from sqlalchemy.exc import SQLAlchemyError

from users_db import config

log = logging.getLogger(__name__)

engine = create_engine(url=config.get_postgres_uri())


def db_connection(f):
    """
    db_connection decorator supports executing select, insert, update, delete
    operations with a single transaction; post-process data in a user friendly manner
    """

    @wraps(f)
    def wrapped(*args, **kwargs):
        with engine.connect() as connection:
            try:
                r = f(*args, **kwargs, db_conn=connection)

                if isinstance(r, (dict, list)):
                    # post-process data here, for example excluding sensitive rows
                    result = r
                elif isinstance(r, (Insert, Update)):
                    result = connection.execute(r.returning(r.table.c.id))
                    result = result.one()[0]
                    connection.commit()
                elif isinstance(r, Delete):
                    result = connection.execute(r)
                    result = result.rowcount
                    connection.commit()

                return result
            except SQLAlchemyError as err:
                log.error(f"Error: {err}")
                connection.rollback()

    return wrapped



import os
import click
import contextlib

import users_db
from users_db import migrations

# We need to change the current working directory to the path where
# users_db is installed.
USERS_DB_ABSPATH = os.path.dirname(os.path.abspath(users_db.__file__))


@contextlib.contextmanager
def cd(new_dir):
    """This is a context manager that changes the current working directory"""
    prev_dir = os.getcwd()
    os.chdir(os.path.expanduser(new_dir))
    try:
        yield
    finally:
        os.chdir(prev_dir)


# The idea to use Click module seen in https://stackoverflow.com/a/74875605


@click.command()
@click.option("-v", "--verbose", is_flag=True, default=False, help="Verbose mode")
def db_current_cmd(verbose):
    """Display current database revision"""
    with cd(USERS_DB_ABSPATH):
        migrations.current(verbose)


@click.command()
@click.option("-r", "--revision", default="head", help="Revision target")
def db_upgrade_cmd(revision):
    """Upgrade to a later database revision"""
    with cd(USERS_DB_ABSPATH):
        migrations.upgrade(revision)


@click.command()
@click.option("-r", "--revision", required=True, help="Revision target")
def db_downgrade_cmd(revision):
    """Revert to a previous database revision"""
    with cd(USERS_DB_ABSPATH):
        migrations.downgrade(revision)
